{******************************************************************************

 ANALOG DEVICES 
 EUROPEAN DSP APPLICATIONS
 
 versatile FLASH utility for embedded systems
  
 Utility module,						   Ver: 0.10.0
 
	created     24-AUG-97 by hs
	last chnage 24-AUG-97 by hs

 ******************************************************************************
}

.module/ram     UTILITY;
.include    	<constant.h>;

.entry		send_16;
.entry  	char2hex;
.entry  	hex2char;
.entry 		ascii_8bit;
.entry 		ascii_16bit;
.entry  	v16bit_ascii;
.entry		inkey_data;
.entry		inkey_ascii;
.entry		wait_cr;

.var/ram/dm 	con_c2h;
.global		con_c2h;

.var/ram/dm	rbit_out;
.global		rbit_out;

.var/ram/dm	obit_in;
.global		obit_in;

.var/ram/dm	inkey;
.global		inkey;


{******************************************************************************
 *
 *  Send null terminated string pointed to by i7.  2 ASCII are in 16-bit.
 *
 *  REGISTER USAGE SUMMARY:
 *
 *  input  : i7
 *  modyfy : none
 *  output : none
 *  destroy: i7, m7 = 1, l7 = 0, ar
 *  keep   : none
 *  memory : out_char_ax1
 *  calls  : none
 *
 ******************************************************************************}
send_16:
        l7 = 0;
        m7 = 1;
again_16:
        ar = dm (i7, m7);
	none = pass ar;
	if eq rts;
	call out_int_ar;	
	jump again_16;
        rts;

{******************************************************************************
 *
 *  Covert a character to a hex value
 *
 *  REGISTER USAGE SUMMARY:
 *
 *  input  : ax1
 *  modyfy : none
 *  output : ar
 *  destroy: ay0, ar
 *  keep   : none
 *  memory : none
 *  calls  : none
 *
 ******************************************************************************}
char2hex:
	ax0 = 0x0;
	dm (con_c2h) = ax0;		{ clear error flag }
	af = tstbit 6 of ax1;		{ check for 0..9 or a..f }
	if ne jump is_a_char;
	ay0 = 0x30;			{ digit }
	ay1 = 0x09;
	ar = ax1 - ay0;			{ remove offset }
	if lt jump er_c2h;		{ input below 0 }
	af = ar - ay1;			{ test for range }
	if gt jump er_c2h;		{ input beyond 9 }
	rts;
is_a_char:
	ay0 = 0x37;
	ay1 = 0x10;
	ar = clrbit 5 of ax1;		{ capitalize incoming data }
	ar = ar - ay0;			{ remove offset }
	if le jump er_c2h;		{ input below A }
	af =  ar - ay1;			{ test for range }
	if ge jump er_c2h;		{ input beyond F }
	rts;

er_c2h:	ax0 = 0x1;
	dm (con_c2h) = ax0;
	rts;

{******************************************************************************
 *
 *  Covert a hex value to a character
 *
 *  REGISTER USAGE SUMMARY:
 *
 *  input  : ax1
 *  modyfy : none
 *  output : ax1
 *  destroy: ay0, ar
 *  keep   : none
 *  memory : none
 *  calls  : none
 *
 ******************************************************************************}
hex2char:
	ay0 = 0x0f;
	ar = ax1 and ay0;
	ax1 = ar;
	ay0 = 0x9;
	ar = ax1 - ay0;
	if le jump h_deci;
	ay0 = 0x57;			{ 0x37 }
	ar = ax1 + ay0;
	ax1 = ar;
	rts;
h_deci:
	ay0 = 0x30;
	ar = ax1 + ay0;
	ax1 = ar;
	rts;


{******************************************************************************
 *
 *  Covert ASCII to a 16bit / 8bit value, no error detection but time out
 *
 *  REGISTER USAGE SUMMARY:
 *
 *  input  : ax1
 *  modify : rbit_out
 *  output : ax1
 *  destroy: ay0, ar, sr0
 *  keep   : none
 *  memory : none
 *  calls  : get_char_ax1, char2hex
 *
 ******************************************************************************}
ascii_8bit:				{ entry point 8bit }
	sr0 = 0;
	dm(rbit_out) = sr0;
	jump a_8bit_1;
 
ascii_16bit:				{ entry point 16bit }
	idle;
	call get_char_ax1_to;		{ get first nibble }
	call char2hex;
	sr0 = 0;
	sr = lshift ar by 12 (lo);	{ combine the nibbles }
	dm(rbit_out) = sr0;

	idle;
	call get_char_ax1_to;		{ get second nibble }
	call char2hex;
	sr0 = dm(rbit_out);
	sr = sr or lshift ar by 8 (lo);	{ combine the nibbles }
	dm(rbit_out) = sr0;

a_8bit_1:
	idle;
	call get_char_ax1_to;		{ get third nibble }
	call char2hex;
	sr0 = dm(rbit_out);
	sr = sr or lshift ar by 4 (lo);	{ combine the nibbles }
	dm(rbit_out) = sr0;

	idle;
	call get_char_ax1_to;		{ get last nibble }
	call char2hex;
	sr0 = dm(rbit_out);
	sr = sr or lshift ar by 0 (lo);	{ combine the nibbles }
	dm(rbit_out) = sr0;
	rts;


{******************************************************************************
 *
 *  Covert a 4 ASCII to a 16bit value and sends them out
 *
 *  REGISTER USAGE SUMMARY:
 *
 *  input  : i4 points to 16bit input variable
 *  modify : none
 *  output : ax1
 *  destroy: ay0, ar, sr0
 *  keep   : none
 *  memory : none
 *  calls  : get_char_ax1, char2hex
 *
 ******************************************************************************}
v16bit_ascii:

	sr0 = dm(obit_in);
	sr = lshift sr0 by -12 (lo);
	ax1 = sr0;
	call hex2char;
	call out_char_ax1;
	
	sr0 = dm(obit_in);
	sr = lshift sr0 by -8 (lo);
	ax1 = sr0;
	call hex2char;
	call out_char_ax1;
	
	sr0 = dm(obit_in);
	sr = lshift sr0 by -4 (lo);
	ax1 = sr0;
	call hex2char;
	call out_char_ax1;

	ax1 = dm(obit_in);
	call hex2char;
	call out_char_ax1;
	rts;

{******************************************************************************
 *
 *  Inkey 4 ASCII and convert to a 16bit value
 *
 *  REGISTER USAGE SUMMARY:
 *
 *  input  : ax1
 *  modify : 16bit_out
 *  output : dm(inkey);
 *  destroy: ay0, ar, sr0
 *  keep   : none
 *  memory : none
 *  calls  : get_char_ax1, char2hex
 *
 ******************************************************************************}
inkey_data:
	
	idle;
	call get_char_ax1;			{ Dxxx nibble }
	call out_char_ax1;
	call char2hex;
	sr = lshift ar by 12 (lo);
	dm (inkey) = sr0;
	ar = dm(con_c2h);
	none = pass ar;
	if ne jump inkey_err;

	idle;
	call get_char_ax1;			{ xDxx nibble }
	call out_char_ax1;
	call char2hex;
	sr0 = dm (inkey);
	sr = sr or lshift ar by 8 (lo);
	dm (inkey) = sr0;
	ar = dm(con_c2h);
	none = pass ar;
	if ne jump inkey_err;

	idle;
	call get_char_ax1;			{ xxDx nibble }
	call out_char_ax1;
	call char2hex;
	sr0 = dm (inkey);
	sr = sr or lshift ar by 4 (lo);
	dm (inkey) = sr0;
	ar = dm(con_c2h);
	none = pass ar;
	if ne jump inkey_err;

	idle;
	call get_char_ax1;			{ xxxD nibble }
	call out_char_ax1;
	call char2hex;
	sr0 = dm (inkey);
	sr = sr or lshift ar by 0 (lo);
	dm (inkey) = sr0;
	ar = dm(con_c2h);
	none = pass ar;
	if ne jump inkey_err;
	rts;

inkey_err:					{ include here error treatment }
	jump inkey_data;
		

{******************************************************************************
 *
 *  Inkey 4 ASCII and convert to a 16bit value
 *
 *  REGISTER USAGE SUMMARY:
 *
 *  input  : ax1
 *  modify : 16bit_out
 *  output : dm(inkey);
 *  destroy: ay0, ar, sr0
 *  keep   : none
 *  memory : none
 *  calls  : get_char_ax1, char2hex
 *
 ******************************************************************************}
inkey_ascii:
	
	idle;
	call get_char_ax1;			{ Dxxx nibble }
	call out_char_ax1;
	call char2hex;
	sr = lshift ar by 12 (lo);
	dm (inkey) = sr0;
	ar = dm(con_c2h);
	none = pass ar;
	if ne jump inkey_err1;

	idle;
	call get_char_ax1;			{ xDxx nibble }
	call out_char_ax1;
	call char2hex;
	sr0 = dm (inkey);
	sr = sr or lshift ar by 8 (lo);
	dm (inkey) = sr0;
	ar = dm(con_c2h);
	none = pass ar;
	if ne jump inkey_err1;

	idle;
	call get_char_ax1;			{ xxDx nibble }
	call out_char_ax1;
	call char2hex;
	sr0 = dm (inkey);
	sr = sr or lshift ar by 4 (lo);
	dm (inkey) = sr0;
	ar = dm(con_c2h);
	none = pass ar;
	if ne jump inkey_err1;

	idle;
	call get_char_ax1;			{ xxxD nibble }
	call out_char_ax1;
	call char2hex;
	sr0 = dm (inkey);
	sr = sr or lshift ar by 0 (lo);
	dm (inkey) = sr0;
	ar = dm(con_c2h);
	none = pass ar;
	if ne jump inkey_err1;
	rts;

inkey_err1:					{ error treatment: reenter data }
	rts;

{******************************************************************************
 *
 *  FUNCTION: wait for CR from UART
 *
 *  received: 0x0d (CR)
 *
 *  REGISTER USAGE SUMMARY:
 *
 *  input  : none
 *  update : ax1
 *  output : none
 *  destroy: ay1
 *  keep   : none
 *  memory : none
 *  calls  : none
 *
 ******************************************************************************}
wait_cr:
	idle;
	call get_char_ax1;
        ay1 = 0x0d;
        none = ax1 - ay1;
        if eq rts;
	jump wait_cr;

.endmod;
